import React from 'react'
import {
  BrowserRouter as Router,
  Route,
  Link
} from 'react-router-dom'
import './App.css';

/* ####################################################################################################################### */
/* #### PAGES ############################################################################################################ */
/* ####################################################################################################################### */

class Home extends React.Component {
  render() {
    return (
      <div>
        <h1>Accueil</h1>
        <h2>Auteur : Antoine Duval</h2>
        <p>Veuillez choisir votre capteur à gauche.</p>
      </div>
    );
  }
}

class SensorPage extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      sensor: props.data_sensor
    };
  }

  render(){
    switch(this.state.sensor._type){
      case "TEMPERATURE":
        return (
          <div>
            <h1>{this.state.sensor._name}</h1>
            <TemperatureSensor data_sensor={this.state.sensor} />
          </div>
        );
        break;
      case "ON_OFF":
        return (
          <div>
            <h1>{this.state.sensor._name}</h1>
            <BooleanSensor data_sensor={this.state.sensor} />
          </div>
        );
        break;
      case "Percent":
        return (
          <div>
            <h1>{this.state.sensor._name}</h1>
            <PercentSensor data_sensor={this.state.sensor} />
          </div>
        );
        break;
      default:
        return (
          <div>
          </div>
        );
        break;
    }
  }
}

/* ####################################################################################################################### */
/* #### SENSORS ########################################################################################################## */
/* ####################################################################################################################### */

class PercentSensor extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      sensor: props.data_sensor
    };
  }

  render(){
    return (
      <div>
        <div className="col">
          <div className="row">
            <div>Valeur actuelle : </div>
            <div className="big_value"><span>{this.state.sensor.lastPercent()} %</span></div>
          </div>
          <div className="row">
            <div>Historique : </div>
            <div>
              <div className="col sub_table">
                {this.state.sensor._dataSensor.get_last_values(10).map(function(name, index){
                  return (
                    <div key={ index }>
                      <div>{name} %</div>
                    </div>
                  );
                })}
              </div>
            </div>
          </div>
        </div>
      </div>
    );
  }
}

class BooleanSensor extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      sensor: props.data_sensor
    };
  }

  render() {
    return (
      <div>
        <div className="col">
          <div className="row">
            <div>Valeur actuelle : </div>
            <div className="big_value"><span>{this.state.sensor._dataSensor._value}</span></div>
          </div>
        </div>
      </div>
    );
  }
}

class TemperatureSensor extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      sensor: props.data_sensor
    };
  }

  render() {
    let lastMesure = Math.round(this.state.sensor._dataSensor.lastMesure()*100)/100;
    return (
      <div>
        <div className="col">
          <div className="row">
            <div>Valeur actuelle : </div>
            <div className="big_value"><span>{lastMesure}</span>°C</div>
          </div>
          <div className="row">
            <div>Historique : </div>
            <div>
              <div className="col sub_table">
                {this.state.sensor._dataSensor.get_last_values(10).map(function(name, index){
                  return (
                    <div key={ index }>
                      <div>{Math.round(name*100)/100}°C</div>
                    </div>
                  );
                })}
              </div>
            </div>
          </div>
        </div>
      </div>
    );
  }
}

/* ####################################################################################################################### */
/* #### MAIN ############################################################################################################# */
/* ####################################################################################################################### */

class BasicExample extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      values: props.values
    };
  }

  render(){
    return (
      <Router>
        <div className="MyRoute">
          <div className="left_menu">
            <div><Link to="/">Accueil</Link></div>
            {this.state.values.map(function(name, index){
              return (
                <div key={ index }>
                  <Link to={"/"+name._url}>{name._name}</Link>
                </div>
              );
            })}
          </div>
          <div className="right_menu">
            <Route exact path="/" component={Home}/>
            {this.state.values.map(function(name, index){
              return(
                <Route key={ index } path={"/"+name._url} component={() => <SensorPage data_sensor={name} />} />
              );
            })}
          </div>
        </div>
      </Router>
    )
  }
}

class Main extends React.Component{
  constructor(props) {
    super(props);
    this.state = {
      values : props.values
    };
  }

  render(){
    return (
      <BasicExample values={this.state.values} />
    );
  }
}

function random(min, max){
  return Math.floor(Math.random()*(max-min+1)+min);
}

export default Main
